@@ -1,7 +1,4 @@
-import argparse
-import shlex
-import shutil
-import subprocess
+import argparse, shlex, shutil, subprocess
 from pathlib import Path
 
 import ffmpeg
@@ -10,10 +7,9 @@ import sqlite_utils
 from catt.api import CattDevice
 from rich.prompt import Confirm
 
-from xklb import utils
+from xklb import paths, utils
 from xklb.db import sqlite_con
-from xklb.utils import SC, cmd, log
-from xklb.utils_player import (
+from xklb.player import (
     delete_media,
     get_ordinal_media,
     listen_chromecast,
@@ -27,6 +23,7 @@ from xklb.utils_player import (
     socket_play,
     watch_chromecast,
 )
+from xklb.utils import SC, cmd, log
 
 idle_mpv = lambda args: ["mpv", "--idle", f"--input-ipc-server={args.mpv_socket}"]
 
@@ -36,9 +33,18 @@ def parse_args(action, default_db, default_chromecast=""):
         prog="lb " + action,
         usage=f"""lb {action} [database] [optional args]
 
+    Control playback:
+
+        To stop playback press Ctrl-C in either the terminal or mpv
+
+        Create global shortcuts in your desktop environment by sending commands to mpv_socket:
+
+        echo 'playlist-next force' | socat - /tmp/mpv_socket
+
     If not specified, {action} will try to read {default_db} in the working directory:
 
         lb {action}
+        lb {action} ./my/other/database/is-a/db.db
 
     Override the default player (mpv):
 
@@ -256,8 +262,9 @@ def parse_args(action, default_db, default_chromecast=""):
 
     # TODO: maybe try https://dba.stackexchange.com/questions/43415/algorithm-for-finding-the-longest-prefix
     parser.add_argument("--play-in-order", "-O", action="count", default=0, help=argparse.SUPPRESS)
-    parser.add_argument("--duration", "-d", action="append", help=argparse.SUPPRESS)
     parser.add_argument("--sort", "-u", nargs="+", help=argparse.SUPPRESS)
+    parser.add_argument("--random", "-r", action="store_true", help=argparse.SUPPRESS)
+
     parser.add_argument("--where", "-w", nargs="+", action="extend", default=[], help=argparse.SUPPRESS)
     parser.add_argument("--include", "-s", "--search", nargs="+", action="extend", default=[], help=argparse.SUPPRESS)
     parser.add_argument("--exclude", "-E", "-e", nargs="+", action="extend", default=[], help=argparse.SUPPRESS)
@@ -270,6 +277,7 @@ def parse_args(action, default_db, default_chromecast=""):
 
     parser.add_argument("--prefix", default="", help=argparse.SUPPRESS)
 
+    parser.add_argument("--duration", "-d", action="append", help=argparse.SUPPRESS)
     parser.add_argument("--size", "-z", action="append", help=argparse.SUPPRESS)
 
     parser.add_argument("--print", "-p", default=False, const="p", nargs="?", help=argparse.SUPPRESS)
@@ -282,7 +290,7 @@ def parse_args(action, default_db, default_chromecast=""):
     parser.add_argument("--start", "-vs", help=argparse.SUPPRESS)
     parser.add_argument("--end", "-ve", help=argparse.SUPPRESS)
     parser.add_argument("--player", "-player", help=argparse.SUPPRESS)
-    parser.add_argument("--mpv-socket", default=utils.DEFAULT_MPV_SOCKET, help=argparse.SUPPRESS)
+    parser.add_argument("--mpv-socket", default=paths.DEFAULT_MPV_SOCKET, help=argparse.SUPPRESS)
 
     parser.add_argument(
         "--player-args-when-sub", "-player-sub", nargs="*", default=["--speed=1"], help=argparse.SUPPRESS
@@ -310,8 +318,23 @@ def parse_args(action, default_db, default_chromecast=""):
     elif args.limit in ["inf", "all"]:
         args.limit = None
 
-    if not args.sort and args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch]:
-        args.sort = ["priority"]
+    if not args.sort:
+        if args.action in [SC.listen, SC.watch]:
+            args.sort = ["priority"]
+            if args.print and args.include:
+                args.sort = ["duration"]
+
+        elif args.action in [SC.tubelisten, SC.tubewatch]:
+            args.sort = ["play_count, random"]
+            if args.print and args.include:
+                args.sort = ["playlist_path, duration"]
+
+        elif args.action in [SC.filesystem]:
+            args.sort = ["sparseness, size"]
+
+    if args.random:
+        args.sort = ["random"]
+
     if args.sort:
         args.sort = " ".join(args.sort)
         args.sort = override_sort(args.sort)
@@ -491,10 +514,10 @@ def play(args, media: pd.DataFrame):
             if args.transcode:
                 media_file = transcode(media_file)
 
-        if args.action in [SC.watch, SC.tubewatch]:
-            print(media_file)
-        elif args.action == SC.listen:
+        if args.action == SC.listen:
             print(cmd("ffprobe", "-hide_banner", "-loglevel", "info", media_file).stderr)
+        else:
+            print(media_file)
 
         if args.chromecast:
             chromecast_play(args, m)
@@ -578,6 +601,7 @@ def construct_fs_query(args):
             bindings.append("%" + exc.replace(" ", "%").replace("%%", " ") + "%")
 
     args.sql_filter = " ".join(cf)
+    args.sql_filter_bindings = bindings
 
     LIMIT = "LIMIT " + str(args.limit) if args.limit else ""
     OFFSET = f"OFFSET {args.skip}" if args.skip else ""
@@ -629,7 +653,7 @@ def process_actions(args, construct_query=construct_fs_query):
             utils.pkill(idle_mpv(args), strict=False)
         Path(args.mpv_socket).unlink(missing_ok=True)
         if args.chromecast:
-            Path(utils.CAST_NOW_PLAYING).unlink(missing_ok=True)
+            Path(paths.CAST_NOW_PLAYING).unlink(missing_ok=True)
 
 
 def watch():
