@@ -1,10 +1,6 @@
-import argparse
-import math
-import os
-import sys
+import argparse, math, os, sys
 from pathlib import Path
 from shutil import which
-from sqlite3 import OperationalError
 from typing import Dict
 
 import ffmpeg
@@ -13,20 +9,11 @@ import pandas as pd
 from joblib import Parallel, delayed
 from tinytag import TinyTag
 
-from xklb.db import fetchall_dict, sqlite_con
-from xklb.subtitle import get_subtitle, has_external_subtitle, youtube_dl_id
-from xklb.utils import (
-    SQLITE_PARAM_LIMIT,
-    chunks,
-    cmd,
-    combine,
-    filter_None,
-    get_media_files,
-    log,
-    safe_unpack,
-    single_column_tolist,
-)
-from xklb.utils_player import mark_media_deleted
+from xklb import subtitle, utils
+from xklb.db import fetchall_dict, optimize_db, sqlite_con
+from xklb.paths import get_media_files, youtube_dl_id
+from xklb.player import mark_media_deleted
+from xklb.utils import SQLITE_PARAM_LIMIT, cmd, combine, log, safe_unpack
 
 
 def get_provenance(file):
@@ -98,6 +85,8 @@ def calculate_sparseness(stat):
 
 
 def extract_metadata(args, f):
+    log.debug(f)
+
     try:
         stat = os.stat(f)
     except Exception:
@@ -173,26 +162,16 @@ def extract_metadata(args, f):
                 if s.get("r_frame_rate") is not None and "/0" not in s.get("r_frame_rate")
             ]
         )
-        width = safe_unpack([s.get("width") for s in streams if s.get("tags") is not None])
-        height = safe_unpack([s.get("height") for s in streams if s.get("tags") is not None])
+        width = safe_unpack([s.get("width") for s in streams])
+        height = safe_unpack([s.get("height") for s in streams])
         codec_types = [s.get("codec_type") for s in streams]
-        tags = [s.get("tags") for s in streams if s.get("tags") is not None]
-        language = combine([t.get("language") for t in tags if t.get("language") not in [None, "und", "unk"]])
+        stream_tags = [s.get("tags") for s in streams if s.get("tags") is not None]
+        language = combine([t.get("language") for t in stream_tags if t.get("language") not in [None, "und", "unk"]])
 
         video_count = sum([1 for s in codec_types if s == "video"])
         audio_count = sum([1 for s in codec_types if s == "audio"])
-        attachment_count = sum([1 for s in codec_types if s == "attachment"])
-        subtitle_count = sum([1 for s in codec_types if s == "subtitle"])
         chapter_count = len(probe["chapters"])
 
-        if subtitle_count == 0 and args.db_type == "v":
-            try:
-                has_sub = has_external_subtitle(f)
-            except Exception:
-                has_sub = False
-            if has_sub:
-                subtitle_count = 1
-
         media = {
             **media,
             "is_deleted": 0,
@@ -200,53 +179,49 @@ def extract_metadata(args, f):
             "time_played": 0,
             "video_count": video_count,
             "audio_count": audio_count,
-            "subtitle_count": subtitle_count,
             "chapter_count": chapter_count,
-            "attachment_count": attachment_count,
             "width": width,
             "height": height,
             "fps": fps,
+            "duration": 0 if not duration else int(float(duration)),
             "language": language,
             "provenance": get_provenance(f),
-            "duration": 0 if not duration else int(float(duration)),
         }
 
-    if args.db_type == "a":
-        try:
-            tiny_tags = filter_None(TinyTag.get(f).as_dict())
-        except Exception:
-            tiny_tags = dict()
+        if args.db_type == "v":
+            attachment_count = sum([1 for s in codec_types if s == "attachment"])
+            internal_subtitles = [
+                subtitle.extract(f, s["index"])
+                for s in streams
+                if s.get("codec_type") == "subtitle" and s.get("codec_name") not in subtitle.IMAGE_SUBTITLE_CODECS
+            ]
+            external_subtitles = subtitle.get_external(f)
+            subs_text = subtitle.subs_to_text(f, internal_subtitles + external_subtitles)
 
-        try:
-            mutagen_tags = filter_None(mutagen.File(f).tags.as_dict())
-        except Exception:
-            mutagen_tags = dict()
+            video_tags = {
+                "subtitle_count": len(internal_subtitles + external_subtitles),
+                "attachment_count": attachment_count,
+                "tags": combine(subs_text),
+            }
+            return {**media, **video_tags}
 
-        tags = parse_tags(mutagen_tags, tiny_tags)
-        return {**media, **tags}
+        if args.db_type == "a":
+            try:
+                tiny_tags = utils.filter_None(TinyTag.get(f).as_dict())
+            except Exception:
+                tiny_tags = dict()
+
+            try:
+                mutagen_tags = utils.filter_None(mutagen.File(f).tags.as_dict())
+            except Exception:
+                mutagen_tags = dict()
+
+            stream_tags = parse_tags(mutagen_tags, tiny_tags)
+            return {**media, **stream_tags}
 
     return media
 
 
-def get_columns(args):
-    try:
-        query = "SELECT name FROM PRAGMA_TABLE_INFO('media') where type in ('TEXT', 'INTEGER');"
-        cols = single_column_tolist(args.con.execute(query).fetchall(), "name")
-    except OperationalError:
-        cols = []
-    return cols
-
-
-def optimize_db(args):
-    print("Optimizing database")
-    if Path(args.database).exists():
-        cmd("sqlite-utils", "optimize", args.database)
-        columns = get_columns(args)
-
-        for column in columns:
-            cmd("sqlite-utils", "create-index", "--if-not-exists", "--analyze", args.database, "media", column)
-
-
 def extract_chunk(args, l):
     metadata = (
         Parallel(n_jobs=-1 if args.verbose == 0 else 1, backend="threading")(
@@ -282,7 +257,7 @@ def find_new_files(args, path):
 
     try:
         existing = set(
-            single_column_tolist(
+            utils.single_column_tolist(
                 fetchall_dict(args.con, f"select path from media where is_deleted=0 and path like '{path}%'"), "path"
             )
         )
@@ -301,17 +276,17 @@ def find_new_files(args, path):
 
 def scan_path(args, path):
     path = Path(path).resolve()
-    print(f"{path} : Scanning...")
+    print(f"[{path}] Building file list...")
 
     new_files = find_new_files(args, path)
 
     if len(new_files) > 0:
-        print(f"Adding {len(new_files)} new media")
+        print(f"[{path}] Adding {len(new_files)} new media")
         log.debug(new_files)
 
         batch_count = SQLITE_PARAM_LIMIT // 100
         chunks_count = math.ceil(len(new_files) / batch_count)
-        df_chunked = chunks(new_files, batch_count)
+        df_chunked = utils.chunks(new_files, batch_count)
         for idx, l in enumerate(df_chunked):
             percent = ((batch_count * idx) + len(l)) / len(new_files) * 100
             print(f"[{path}] Extracting metadata {percent:3.1f}% (chunk {idx + 1} of {chunks_count})")
@@ -319,7 +294,7 @@ def scan_path(args, path):
 
             if args.subtitle:
                 print("Fetching subtitles")
-                Parallel(n_jobs=5)(delayed(get_subtitle)(args, file) for file in l)
+                Parallel(n_jobs=5)(delayed(subtitle.get)(args, file) for file in l)
 
     return len(new_files)
 
@@ -347,24 +322,30 @@ def parse_args():
     This will create audio.db in the current directory:
         lb fsadd --audio ./music/
 
+    Run with --optimize to add indexes to every int and text column:
+        lb fsadd --optimize --audio ./music/
+
     The database location must be specified to reference more than one path:
         lb fsadd --audio podcasts.db ./podcasts/ ./another/folder/
 """,
     )
     parser.add_argument("database", nargs="?")
     parser.add_argument("paths", nargs="+")
-    parser.add_argument("--db", "-db")
+    parser.add_argument("--db", "-db", help=argparse.SUPPRESS)
 
     db_type = parser.add_mutually_exclusive_group()
-    db_type.add_argument("--audio", action="store_const", dest="db_type", const="a")
-    db_type.add_argument("--filesystem", action="store_const", dest="db_type", const="f")
-    db_type.add_argument("--video", action="store_const", dest="db_type", const="v")
+    db_type.add_argument("--audio", action="store_const", dest="db_type", const="a", help="Create audio database")
+    db_type.add_argument(
+        "--filesystem", action="store_const", dest="db_type", const="f", help="Create filesystem database"
+    )
+    db_type.add_argument("--video", action="store_const", dest="db_type", const="v", help="Create video database")
     parser.set_defaults(db_type="v")
 
-    parser.add_argument("--subtitle", action="store_true")
-    parser.add_argument("--youtube-only", action="store_true")
-    parser.add_argument("--subliminal-only", action="store_true")
-    parser.add_argument("--delete-unplayable", action="store_true")
+    parser.add_argument("--subtitle", action="store_true", help=argparse.SUPPRESS)
+    parser.add_argument("--youtube-only", action="store_true", help=argparse.SUPPRESS)
+    parser.add_argument("--subliminal-only", action="store_true", help=argparse.SUPPRESS)
+    parser.add_argument("--delete-unplayable", action="store_true", help=argparse.SUPPRESS)
+    parser.add_argument("--optimize", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--verbose", "-v", action="count", default=0)
     args = parser.parse_args()
 
@@ -380,6 +361,9 @@ def parse_args():
             args.database = "video.db"
         else:
             raise Exception(f"fs_extract for db_type {args.db_type} not implemented")
+
+    log.info(utils.filter_None(args.__dict__))
+
     return args
 
 
