@@ -1,26 +1,67 @@
-import argparse
-import os
-import re
+import argparse, os, re, tempfile
 from pathlib import Path
 from shlex import quote
+from typing import List
 
+import ffmpeg, pysubs2
+from ffmpeg import Error
 from joblib import Parallel, delayed
 
-from xklb.utils import cmd, get_media_files
+from xklb.paths import get_media_files, youtube_dl_id
+from xklb.utils import cmd, flatten, log, remove_text_inside_brackets, remove_whitespaace
+
+SUBTITLE_FORMATS = "vtt|srt|ssa|ass|jss|aqt|mpl2|mpsub|pjs|rt|sami|smi|stl|xml|txt|psb|ssf|usf"
+IMAGE_SUBTITLE_CODECS = ["dvbsub", "dvdsub", "pgssub", "xsub", "dvb_subtitle", "dvd_subtitle", "hdmv_pgs_subtitle"]
 
 
-def youtube_dl_id(file) -> str:
-    if len(file) < 15:
-        return ""
-    # rename old youtube_dl format to new one: cargo install renamer; fd -tf . -x renamer '\-([\w\-_]{11})\.= [$1].' {}
-    yt_id_regex = re.compile(r"-([\w\-_]{11})\..*$|\[([\w\-_]{11})\]\..*$", flags=re.M)
-    file = str(file).strip()
+def extract(video_file, stream_index):
+    temp_vtt = tempfile.mktemp(".vtt")
 
-    yt_ids = yt_id_regex.findall(file)
-    if len(yt_ids) == 0:
-        return ""
+    ffmpeg.input(video_file).output(temp_vtt, map="0:" + str(stream_index)).run(quiet=True)
+    return temp_vtt
 
-    return list(filter(None, [*yt_ids[0]]))[0]
+
+def convert_to_srt(path):
+    temp_srt = tempfile.mktemp(".srt")
+    try:
+        ffmpeg.input(path).output(temp_srt).run(quiet=True)
+    except Error as e:
+        log.warning(e.args)
+        log.warning(e.stdout)
+        log.warning(e.stderr)
+        log.error(e.__traceback__)
+        raise UnicodeDecodeError("utf-8", b"Dr. John A. Zoidberg", 1, 2, "Bleh!")
+    else:
+        return temp_srt
+
+
+def read_sub_unsafe(path):
+    return [
+        remove_text_inside_brackets(caption.text.replace(r"\N", " ").replace(r"\n", " ").replace("\n", " "))
+        for caption in pysubs2.load(path, format_="srt")
+    ]
+
+
+def read_sub(path):
+    if Path(path).suffix != ".srt":
+        path = convert_to_srt(path)
+
+    try:
+        return read_sub_unsafe(path)
+    except UnicodeDecodeError:
+        return read_sub_unsafe(convert_to_srt(path))
+
+
+def subs_to_text(video_path, paths: List[str]):
+    def sub_to_text(path):
+        try:
+            return read_sub(path)
+        except UnicodeDecodeError:
+            log.warning(f"[{video_path}] Could not decode subtitle {path}")
+            return []
+
+    subtitles = " ".join(list(dict.fromkeys(flatten([sub_to_text(path) for path in paths]))))
+    return remove_whitespaace(subtitles)
 
 
 def has_internal_subtitle(file):
@@ -33,8 +74,22 @@ def has_internal_subtitle(file):
         return True
 
 
+def get_external(file):
+    p = Path(file)
+    for suffix in reversed(p.suffixes):
+        subtitles = [
+            str(sub_p)
+            for sub_p in p.parent.glob(p.stem[: -len(suffix)] + "*")
+            if sub_p.suffix[1:] in SUBTITLE_FORMATS.split("|")
+        ]
+
+        if len(subtitles) > 0:
+            return subtitles
+
+    return []
+
+
 def has_external_subtitle(file):
-    SUBTITLE_FORMATS = "vtt|srt|ssa|ass|sub|idx|psb|smi|ssf|usf"
     file = Path(file)
 
     if any(
@@ -59,7 +114,7 @@ def has_external_subtitle(file):
     return False
 
 
-def get_subtitle(args, file):
+def get(args, file):
     try:
         if has_internal_subtitle(file) or has_external_subtitle(file):
             return
@@ -112,7 +167,7 @@ def main():
 
     video_files = get_media_files(args)
 
-    Parallel(n_jobs=6 if args.verbose == 0 else 1)(delayed(get_subtitle)(args, file) for file in video_files)
+    Parallel(n_jobs=6 if args.verbose == 0 else 1)(delayed(get)(args, file) for file in video_files)
 
 
 if __name__ == "__main__":
