@@ -4,9 +4,9 @@ from time import sleep
 import pandas as pd
 
 from xklb.db import sqlite_con
+from xklb.player import generic_player, mark_media_watched, override_sort, printer
 from xklb.tabs_extract import Frequency
 from xklb.utils import SC, cmd, filter_None, flatten, log
-from xklb.utils_player import generic_player, mark_media_watched, override_sort, printer
 
 tabs_include_string = (
     lambda x: f"""and (
@@ -66,7 +66,12 @@ def construct_tabs_query(args):
         , frequency = 'quarterly' desc
         , frequency = 'yearly' desc
         {', path' if args.print else ''}
-        , ROW_NUMBER() OVER ( PARTITION BY category ) -- prefer to spread categories over time
+        , ROW_NUMBER() OVER ( PARTITION BY
+            play_count
+            , frequency
+            , hostname
+            , category
+        ) -- prefer to spread hostname, category over time
         , random()
     {LIMIT} {OFFSET}
     """
@@ -120,8 +125,50 @@ def process_tabs_actions(args, construct_query):
     play(args, media)
 
 
-def parse_args(action, default_db, default_chromecast=""):
-    parser = argparse.ArgumentParser(prog="lb tabs", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+def parse_args(action, default_db):
+    parser = argparse.ArgumentParser(
+        prog="lb tabs",
+        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
+        usage="""lb tabs [database] [optional args]
+
+    lb tabs is meant to run **once per day**. Here is how you would configure it with `crontab`:
+
+        45 9 * * * DISPLAY=:0 lb tabs /home/my/tabs.db
+
+    If things aren't working you can use `at` to simulate a similar environment as `cron`
+
+        echo 'fish -c "export DISPLAY=:0 && lb tabs /full/path/to/tabs.db"' | at NOW
+
+    You can also invoke tabs manually:
+
+        lb tabs -L 1  # open one tab
+
+    Print URLs
+
+        lb-dev tabs -w "frequency='yearly'" -p
+        ╒════════════════════════════════════════════════════════════════╤═════════════╤══════════════╕
+        │ path                                                           │ frequency   │ time_valid   │
+        ╞════════════════════════════════════════════════════════════════╪═════════════╪══════════════╡
+        │ https://old.reddit.com/r/Autonomia/top/?sort=top&t=year        │ yearly      │ Dec 31 1970  │
+        ├────────────────────────────────────────────────────────────────┼─────────────┼──────────────┤
+        │ https://old.reddit.com/r/Cyberpunk/top/?sort=top&t=year        │ yearly      │ Dec 31 1970  │
+        ├────────────────────────────────────────────────────────────────┼─────────────┼──────────────┤
+        │ https://old.reddit.com/r/ExperiencedDevs/top/?sort=top&t=year  │ yearly      │ Dec 31 1970  │
+
+        ...
+
+        ╘════════════════════════════════════════════════════════════════╧═════════════╧══════════════╛
+
+    View how many yearly tabs you have:
+
+        lb-dev tabs -w "frequency='yearly'" -p a
+        ╒═══════════╤═════════╕
+        │ path      │   count │
+        ╞═══════════╪═════════╡
+        │ Aggregate │     134 │
+        ╘═══════════╧═════════╛
+""",
+    )
     parser.add_argument(
         "database",
         nargs="?",
@@ -129,59 +176,15 @@ def parse_args(action, default_db, default_chromecast=""):
         help="Database file. If not specified a generic name will be used: audio.db, video.db, fs.db, etc",
     )
 
-    parser.add_argument(
-        "--sort",
-        "-u",
-        nargs="+",
-        help="""Sort media with SQL expressions
--u duration means shortest media first
--u duration desc means longest media first
+    parser.add_argument("--sort", "-u", nargs="+")
+    parser.add_argument("--where", "-w", nargs="+", action="extend", default=[])
+    parser.add_argument("--include", "-s", "--search", nargs="+", action="extend", default=[])
+    parser.add_argument("--exclude", "-E", "-e", nargs="+", action="extend", default=[])
 
-You can use any sqlite ORDER BY expressions, for example:
--u subtitle_count > 0
-means play everything that has a subtitle first
-""",
-    )
-    parser.add_argument(
-        "--where",
-        "-w",
-        nargs="+",
-        action="extend",
-        default=[],
-        help="""Constrain media with SQL expressions
-You can use any sqlite WHERE expressions, for example:
--w attachment_count > 0  means only media with attachments
--w language = 'eng'  means only media which has some English language tag -- this could be audio or subtitle""",
-    )
-    parser.add_argument(
-        "--include",
-        "-s",
-        "--search",
-        nargs="+",
-        action="extend",
-        default=[],
-        help="""Constrain media with via search
--s toy story will match '/folder/toy/something/story.mp3'
--s 'toy  story' will match more strictly '/folder/toy story.mp3'
-Double spaces means one space
-""",
-    )
-    parser.add_argument("--exclude", "-E", "-e", nargs="+", action="extend", default=[], help="--include but opposite")
-
-    parser.add_argument(
-        "--print",
-        "-p",
-        default=False,
-        const="p",
-        nargs="?",
-        help="""Print instead of play
--p   means print in a table
--p a means print an aggregate report
--p f means print only filenames -- useful for piping to other utilities like xargs or GNU Parallel""",
-    )
+    parser.add_argument("--print", "-p", default=False, const="p", nargs="?")
     parser.add_argument("--cols", "-cols", "-col", nargs="*", help="Include a non-standard column when printing")
-    parser.add_argument("--limit", "-L", "-l", "-queue", "--queue", help="Set play queue size")
-    parser.add_argument("--skip", "-S", help="Offset from the top of an ordered query; wt -S10 to skip ten videos")
+    parser.add_argument("--limit", "-L", "-l", "-queue", "--queue")
+    parser.add_argument("--skip", "-S")
 
     parser.add_argument("--db", "-db")
     parser.add_argument("--verbose", "-v", action="count", default=0)
