@@ -6,19 +6,16 @@ from tabulate import tabulate
 
 from xklb.db import sqlite_con
 from xklb.fs_actions import parse_args, process_actions
+from xklb.player import delete_playlists
 from xklb.utils import SC, filter_None, human_time, log, resize_col
-from xklb.utils_player import delete_playlists
 
 # TODO: add cookiesfrombrowser: ('firefox', ) as a default
 # cookiesfrombrowser: ('vivaldi', ) # should not crash if not installed ?
 
 default_ydl_opts = {
-    # "writesubtitles": True,
-    # "writeautomaticsub": True,
-    # "subtitleslangs": "en.*,EN.*",
+    "extract_flat": True,
     "lazy_playlist": True,
     "skip_download": True,
-    "force_write_download_archive": True,
     "check_formats": False,
     "no_check_certificate": True,
     "no_warnings": True,
@@ -29,7 +26,6 @@ default_ydl_opts = {
     "dynamic_mpd": False,
     "youtube_include_dash_manifest": False,
     "youtube_include_hls_manifest": False,
-    "extract_flat": True,
     "clean_infojson": False,
     "playlistend": 20000,
     "rejecttitle": "|".join(
@@ -172,35 +168,53 @@ def printer(args):
         print(tabulate(tbl, tablefmt="fancy_grid", headers="keys", showindex=False))  # type: ignore
 
         if "duration" in db_resp.columns:
-            print(f"{len(db_resp)} items")
+            print(f"{len(db_resp)} playlists" if len(db_resp) > 1 else "1 playlist")
             summary = db_resp.sum(numeric_only=True)
             duration = summary.get("duration") or 0
             print("Total duration:", human_time(duration))
 
 
 def tube_list():
-    parser = argparse.ArgumentParser(prog="lb tubelist")
+    parser = argparse.ArgumentParser(
+        prog="lb tubelist",
+        usage="""lb tubelist [database] [--print {p,f,a}] [--delete ...]
+
+    List of Playlists
+
+        lb tubelist
+        ╒══════════╤════════════════════╤══════════════════════════════════════════════════════════════════════════╕
+        │ ie_key   │ title              │ path                                                                     │
+        ╞══════════╪════════════════════╪══════════════════════════════════════════════════════════════════════════╡
+        │ Youtube  │ Highlights of Life │ https://www.youtube.com/playlist?list=PL7gXS9DcOm5-O0Fc1z79M72BsrHByda3n │
+        ╘══════════╧════════════════════╧══════════════════════════════════════════════════════════════════════════╛
+
+    Aggregate Report of Videos in each Playlist
+
+        lb tubelist -p a
+        ╒══════════╤════════════════════╤══════════════════════════════════════════════════════════════════════════╤═══════════════╤═════════╕
+        │ ie_key   │ title              │ path                                                                     │ duration      │   count │
+        ╞══════════╪════════════════════╪══════════════════════════════════════════════════════════════════════════╪═══════════════╪═════════╡
+        │ Youtube  │ Highlights of Life │ https://www.youtube.com/playlist?list=PL7gXS9DcOm5-O0Fc1z79M72BsrHByda3n │ 53.28 minutes │      15 │
+        ╘══════════╧════════════════════╧══════════════════════════════════════════════════════════════════════════╧═══════════════╧═════════╛
+        1 playlist
+        Total duration: 53.28 minutes
+
+    Print only playlist urls:
+
+        Useful for piping to other utilities like xargs or GNU Parallel.
+
+        lb tubelist -p f
+        https://www.youtube.com/playlist?list=PL7gXS9DcOm5-O0Fc1z79M72BsrHByda3n
+
+    Remove a playlist/channel and all linked videos:
+
+        lb tubelist --remove https://vimeo.com/canal180
+""",
+    )
     parser.add_argument("database", nargs="?", default="tube.db")
-    parser.add_argument("--db", "-db")
-    parser.add_argument(
-        "--print",
-        "-p",
-        nargs="*",
-        default="p",
-        choices=["p", "f", "a"],
-        help="""tubelist -p a -- means print playlists in a table
-tubelist -p a -- means print an aggregate report
-tubelist -p f -- means print only playlist urls -- useful for piping to other utilities like xargs or GNU Parallel""",
-    )
-    parser.add_argument(
-        "--delete",
-        "--remove",
-        "--erase",
-        "--rm",
-        "-rm",
-        nargs="+",
-        help="""lb tubelist -rm https://vimeo.com/canal180 -- removes the playlist/channel and all linked videos""",
-    )
+    parser.add_argument("--db", "-db", help=argparse.SUPPRESS)
+    parser.add_argument("--print", "-p", nargs="*", default="p", choices=["p", "f", "a"], help=argparse.SUPPRESS)
+    parser.add_argument("--delete", "--remove", "--erase", "--rm", "-rm", nargs="+", help=argparse.SUPPRESS)
     parser.add_argument("-v", "--verbose", action="count", default=0)
     args = parser.parse_args()
     log.info(filter_None(args.__dict__))
