@@ -1,25 +1,28 @@
-import argparse
-import sys
+import argparse, sys, tempfile
 from copy import deepcopy
 from datetime import datetime
 from pathlib import Path
 from sqlite3 import OperationalError
+from time import sleep
 from timeit import default_timer as timer
 from typing import Dict, List, Union
+from urllib.error import HTTPError
 
 import pandas as pd
 import yt_dlp
 
-from xklb.db import sqlite_con
-from xklb.fs_extract import optimize_db
+from xklb import utils
+from xklb.db import optimize_db, sqlite_con
+from xklb.paths import sanitize_url
+from xklb.subtitle import subs_to_text
 from xklb.tube_actions import default_ydl_opts
-from xklb.utils import argparse_dict, combine, filter_None, log, safe_unpack, single_column_tolist
+from xklb.utils import combine, log, safe_unpack
 
 
-def parse_args(action):
-    parser = argparse.ArgumentParser(prog="lb tube" + action)
+def parse_args(action, usage):
+    parser = argparse.ArgumentParser(prog="lb tube" + action, usage=usage)
     parser.add_argument("database", nargs="?", default="tube.db")
-    parser.add_argument("--db", "-db")
+    parser.add_argument("--db", "-db", help=argparse.SUPPRESS)
     if action == "add":
         parser.add_argument("playlists", nargs="+")
     elif action == "update":
@@ -30,17 +33,19 @@ def parse_args(action):
         "--yt-dlp-config",
         "-yt-dlp-config",
         nargs=1,
-        action=argparse_dict,
+        action=utils.argparse_dict,
         default={},
         metavar="KEY=VALUE",
         help="Add key/value pairs to override or extend default yt-dlp configuration",
     )
     parser.add_argument("--safe", "-safe", action="store_true", help="Skip generic URLs")
+    parser.add_argument("--no-sanitize", "-s", action="store_false", help="Don't sanitize some common URL parameters")
+
+    parser.add_argument("--extra", "-extra", action="store_true", help="Get full metadata (takes a lot longer)")
 
     parser.add_argument("--verbose", "-v", action="count", default=0)
     args = parser.parse_args()
     args.action = action
-    log.info(filter_None(args.__dict__))
 
     if args.db:
         args.database = args.db
@@ -49,9 +54,15 @@ def parse_args(action):
     args.con = sqlite_con(args.database)
 
     ydl_opts = {**default_ydl_opts, **args.yt_dlp_config}
-    log.info(filter_None(ydl_opts))
+    log.info(utils.filter_None(ydl_opts))
+
+    if args.playlists and not args.no_sanitize:
+        args.playlists = [sanitize_url(args, path) for path in args.playlists]
 
     args.ydl_opts = ydl_opts
+
+    log.info(utils.filter_None(args.__dict__))
+
     return args
 
 
@@ -63,7 +74,29 @@ def supported(url):  # thank you @dbr
     return False
 
 
-def consolidate(playlist_path, v):
+def get_subtitle_text(ydl: yt_dlp.YoutubeDL, video_path, req_sub_dict):
+    def dl_sub(url):
+        temp_file = tempfile.mktemp(".srt")
+
+        try:
+            ydl.dl(temp_file, {"url": url}, subtitle=True)
+        except HTTPError:
+            log.info("Unable to download subtitles; skipping")
+            sleep(5)
+            return None
+        else:
+            return temp_file
+
+    urls = [d["url"] for d in list(req_sub_dict.values())]
+    paths: List[str] = list(filter(bool, [dl_sub(url) for url in urls]))  # type: ignore
+
+    subs_text = subs_to_text(video_path, paths)
+    [Path(p).unlink(missing_ok=True) for p in paths]
+
+    return subs_text
+
+
+def consolidate(ydl: yt_dlp.YoutubeDL, playlist_path, v):
     ignore_keys = [
         "thumbnail",
         "thumbnails",
@@ -84,7 +117,6 @@ def consolidate(playlist_path, v):
         "display_id",
         "fulltitle",
         "duration_string",
-        "requested_subtitles",
         "format",
         "format_id",
         "ext",
@@ -103,7 +135,6 @@ def consolidate(playlist_path, v):
         "license",
         "timestamp",
         "track",
-        "subtitles",
         "comments",
         "author",
         "text",
@@ -133,6 +164,14 @@ def consolidate(playlist_path, v):
         "playlist_title",
         "playlist_uploader",
         "playlist_uploader_id",
+        "audio_channels",
+        "subtitles",
+        "automatic_captions",
+        "quality",
+        "has_drm",
+        "language_preference",
+        "preference",
+        "location",
     ]
 
     if v.get("title") in ["[Deleted video]", "[Private video]"]:
@@ -142,10 +181,15 @@ def consolidate(playlist_path, v):
         if k.startswith("_") or k in ignore_keys:
             v.pop(k, None)
 
-    upload_date = v.pop("upload_date", None)
+    release_date = v.pop("release_date", None)
+    upload_date = v.pop("upload_date", None) or release_date
     if upload_date:
         upload_date = int(datetime.strptime(upload_date, "%Y%m%d").timestamp())
 
+    subtitles = v.pop("requested_subtitles", None)
+    if subtitles:
+        subtitles = get_subtitle_text(ydl, playlist_path, subtitles)
+
     cv = dict()
     cv["path"] = safe_unpack(v.pop("webpage_url", None), v.pop("url", None), v.pop("original_url", None))
     cv["size"] = v.pop("filesize_approx", None)
@@ -155,7 +199,7 @@ def consolidate(playlist_path, v):
     cv["play_count"] = 0
     cv["time_played"] = 0
     cv["language"] = v.pop("language", None)
-    cv["tags"] = combine(v.pop("description", None), v.pop("categories", None), v.pop("tags", None))
+    cv["tags"] = combine(v.pop("description", None), v.pop("categories", None), v.pop("tags", None), subtitles)
     cv["id"] = v.pop("id")
     cv["ie_key"] = safe_unpack(v.pop("ie_key", None), v.pop("extractor_key", None), v.pop("extractor", None))
     cv["title"] = safe_unpack(v.pop("title", None), v.get("playlist_title"))
@@ -243,7 +287,7 @@ def process_playlist(args, playlist_path) -> Union[List[Dict], None]:
             return [], info
 
         def _add_media(self, entry):
-            entry = consolidate(playlist_path, entry)
+            entry = consolidate(super, playlist_path, entry)  # type: ignore
             if entry:
                 if video_known(args, playlist_path, entry["path"]):
                     raise ExistingPlaylistVideoReached
@@ -278,17 +322,70 @@ def process_playlist(args, playlist_path) -> Union[List[Dict], None]:
                 log_problem(args, playlist_path)
 
 
+def get_extra_metadata(args, playlist_path) -> Union[List[Dict], None]:
+    with yt_dlp.YoutubeDL(
+        {
+            **args.ydl_opts,
+            "subtitlesformat": "srt/best",
+            "writesubtitles": True,
+            "writeautomaticsub": True,
+            "subtitleslangs": ["en.*", "EN.*"],
+            "extract_flat": False,
+            "lazy_playlist": False,
+            "skip_download": True,
+            "check_formats": False,
+            "ignoreerrors": True,
+        }
+    ) as ydl:
+        videos = args.con.execute(
+            """
+                select path, ie_key, play_count, time_played from media
+                where
+                    width is null
+                    and path not like '%playlist%'
+                    and playlist_path = ?
+                order by random()
+                """,
+            [playlist_path],
+        ).fetchall()
+
+        current_video_count = 0
+        for path, ie_key, play_count, time_played in videos:
+            entry = ydl.extract_info(path, ie_key=ie_key, download=False)
+            if entry is None:
+                continue
+
+            entry = consolidate(ydl, playlist_path, entry)
+            if entry is None:
+                continue
+
+            entry["play_count"] = play_count
+            entry["time_played"] = time_played
+
+            args.con.execute("DELETE FROM media where path = ? and ie_key = ?", [path, ie_key])
+            save_entries(args, [entry])
+
+            current_video_count += 1
+            sys.stdout.write("\033[K\r")
+            print(
+                f"{playlist_path}: {current_video_count} of {len(videos)} extra metadata fetched", end="\r", flush=True
+            )
+
+
 def get_playlists(args, include_playlistless_media=True):
     try:
         if include_playlistless_media:
-            known_playlists = single_column_tolist(args.con.execute("select path from playlists").fetchall(), "path")
+            known_playlists = utils.single_column_tolist(
+                args.con.execute("select path from playlists order by random()").fetchall(), "path"
+            )
         else:
-            known_playlists = single_column_tolist(
+            known_playlists = utils.single_column_tolist(
                 args.con.execute(
                     """
                     select playlist_path from media
                     group by playlist_path
                     having count(playlist_path) > 1
+                    order by random()
                     """
                 ).fetchall(),
                 "playlist_path",
@@ -299,7 +396,22 @@ def get_playlists(args, include_playlistless_media=True):
 
 
 def tube_add():
-    args = parse_args("add")
+    args = parse_args(
+        "add",
+        usage="""lb tubeadd [database] playlists ...
+
+    Create a tube database / add playlists or videos to an existing database
+
+        lb tubeadd educational.db https://www.youtube.com/c/BranchEducation/videos
+
+    Fetch extra metadata:
+
+        By default tubeadd will quickly add media.
+        You can always fetch more metadata later via tubeupdate.
+
+        lb tubeupdate tw.db --extra
+""",
+    )
     known_playlists = get_playlists(args)
 
     for playlist in args.playlists:
@@ -316,9 +428,36 @@ def tube_add():
         end = timer()
         log.info(f"{end - start:.1f} seconds to add new playlist and fetch videos")
 
+        if args.extra:
+            log.warning("Getting extra metadata")
+            get_extra_metadata(args, playlist)
+
 
 def tube_update():
-    args = parse_args("update")
+    args = parse_args(
+        "update",
+        usage="""usage: lb tubeupdate [--optimize] [database] [playlists ...]
+
+    Fetch the latest videos from every playlist in your database
+
+        lb tubeupdate educational.db
+
+    Or limit to specific ones...
+
+        lb tubeupdate educational.db https://www.youtube.com/channel/UCBsEUcR-ezAuxB2WlfeENvA/videos ...
+
+    Run with --optimize to add indexes (might speed up searching but the size will increase):
+
+        lb tubeupdate --optimize examples/music.tl.db ''
+
+    Fetch extra metadata:
+
+        By default tubeupdate will quickly add media.
+        You can run with --extra to fetch more details: (best resolution width, height, subtitle tags, etc)
+
+        lb tubeupdate educational.db --extra https://www.youtube.com/channel/UCBsEUcR-ezAuxB2WlfeENvA/videos
+""",
+    )
     known_playlists = get_playlists(args)
 
     for playlist in args.playlists or get_playlists(args, include_playlistless_media=False):
@@ -332,5 +471,8 @@ def tube_update():
         end = timer()
         log.info(f"{end - start:.1f} seconds to update playlist")
 
-    if args.optimize:
-        optimize_db(args)
+        if args.extra:
+            log.warning("Getting extra metadata")
+            get_extra_metadata(args, playlist)
+
+    optimize_db(args)
